@@ -737,6 +737,9 @@ eog_image_set_xmp_data (EogImage *img, EogMetadataReader *md_reader)
 
 	priv = img->priv;
 
+	if (priv->xmp) {
+		xmp_free (priv->xmp);
+	}
 	priv->xmp = eog_metadata_reader_get_xmp_data (md_reader);
 }
 #endif
@@ -751,7 +754,12 @@ eog_image_set_exif_data (EogImage *img, EogMetadataReader *md_reader)
 	priv = img->priv;
 
 #ifdef HAVE_EXIF
+	g_mutex_lock (priv->status_mutex);
+	if (priv->exif) {
+		exif_data_unref (priv->exif);
+	}
 	priv->exif = eog_metadata_reader_get_exif_data (md_reader);
+	g_mutex_unlock (priv->status_mutex);
 
 	priv->exif_chunk = NULL;
 	priv->exif_chunk_len = 0;
@@ -771,6 +779,9 @@ eog_image_set_exif_data (EogImage *img, EogMetadataReader *md_reader)
 		}
 	}
 #else
+	if (priv->exif_chunk) {
+		g_free (priv->exif_chunk);
+	}
 	eog_metadata_reader_get_exif_chunk (md_reader,
 					    &priv->exif_chunk,
 					    &priv->exif_chunk_len);
@@ -1326,7 +1337,7 @@ tmp_file_move_to_uri (EogImage *image,
 		      GError **error)
 {
 	gboolean result;
-	GError **ioerror = NULL;
+	GError *ioerror = NULL;
 
 	result = g_file_move (tmpfile,
 			      file,
@@ -1335,20 +1346,20 @@ tmp_file_move_to_uri (EogImage *image,
 			      NULL,
 			      (GFileProgressCallback) transfer_progress_cb,
 			      image,
-			      ioerror);
+			      &ioerror);
 
 	if (result == FALSE) {
-		if (g_error_matches (*ioerror, G_IO_ERROR,
+		if (g_error_matches (ioerror, G_IO_ERROR,
 				     G_IO_ERROR_EXISTS)) {
 			g_set_error (error, EOG_IMAGE_ERROR,
 				     EOG_IMAGE_ERROR_FILE_EXISTS,
 				     "File exists");
 		} else {
-		g_set_error (error, EOG_IMAGE_ERROR,
-			     EOG_IMAGE_ERROR_VFS,
-			     "VFS error moving the temp file");
+			g_set_error (error, EOG_IMAGE_ERROR,
+				     EOG_IMAGE_ERROR_VFS,
+				     "VFS error moving the temp file");
 		}
-		g_error_free (*ioerror);
+		g_clear_error (&ioerror);
 	}
 
 	return result;
